<?php
class ControllerExtensionPaymentPaybylink extends Controller {

    public function index() {
		$data['button_confirm'] = $this->language->get('button_confirm');
		
		$this->load->model('checkout/order');
        $this->language->load('extension/payment/paybylink');

		$orderInfo = $this->model_checkout_order->getOrder($this->session->data['order_id']);

        $shopId             = intval($this->config->get('payment_paybylink_shopid'));
        $description        = strval('Zamówienie ' . $this->session->data['order_id']);
        $control            = strval($this->session->data['order_id']);
        $email              = strval($orderInfo['email']);
        $price              = floatval($this->currency->format($orderInfo['total'], $orderInfo['currency_code'], $orderInfo['currency_value'], false));
        $notifyURL          = strval(HTTPS_SERVER . 'index.php?route=extension/payment/paybylink/pbl_itn');
        $returnUrlSuccess   = strval(HTTPS_SERVER . 'index.php?route=extension/payment/paybylink/returnOk');
        $hash               = $this->config->get('payment_paybylink_hash');
    
        $data   = $hash . "|" . $shopId . "|" . sprintf("%.2f", $price);
    
        if ($control != null && $control != "") {
            $data .= "|" . $control;
        }
    
        if ($description != null && $description != "") {
            $data .= "|" . $description;
        }
        
        if ($email != null && $email != "") {
            $data .= "|" . $email;
        }
    
        if ($notifyURL != null && $notifyURL != "") {
            $data .= "|" . $notifyURL;
        }
    
        if ($returnUrlSuccess != null && $returnUrlSuccess != "") {
            $data .= "|" . $returnUrlSuccess;
        }
    
        $signature = hash('sha256', $data);
    
        $paybylinkData  = [
            'shopId'            => $shopId,
            'price'             => $price,
            'control'           => $control,
            'description'       => $description,
            'email'             => $email,
            'notifyURL'         => $notifyURL,
            'returnUrlSuccess'  => $returnUrlSuccess,
            'signature'         => $signature
        ];
    
        $data_string = json_encode($paybylinkData);
    
        $ch = curl_init('https://secure.pbl.pl/api/v1/transfer/generate');                                                                      
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');                                                                    
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);                                                                  
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);                                                                      
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(                                                                          
            'Content-Type: application/json',                                                                                
            'Content-Length: ' . strlen($data_string))                                                                      
        );
    
        $result = curl_exec($ch);
        $result = json_decode($result);
    
        if(isset($result->transactionId) AND !empty($result->transactionId)) {
            $payURL = 'https://secure.pbl.pl/transfer/' . $result->transactionId;
        }

		try{
			$order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
			if($order_info['order_status_id'] != $this->config->get('payment_paybylink_order_status_new_id')){
				$message = 'Rozpoczęcie płatności dla zamówienia o nr. '.$this->session->data['order_id'].' w systemie Paybylink';
				$this->model_checkout_order->addOrderHistory($this->session->data['order_id'], $this->config->get('payment_paybylink_order_status_new_id'), $message, false);
			}
		} catch(Exception $e){
			
        }
        
        return $this->load->view('extension/payment/paybylink', array('url' => $payURL));
    }

    public function pbl_itn() {
        header('HTTP/1.1 200 OK');
        header('Content-Type: text/plain');
    
        $response = file_get_contents('php://input');
    
        if(!empty($response)) {
            $response = json_decode($response, true);
    
            $pbl_data = array(
                'transactionId' => $response['transactionId'],
                'control' => $response['control'],
                'email' => $response['email'],
                'amountPaid' => $response['amountPaid'],
                'notificationAttempt' => $response['notificationAttempt'],
                'paymentType' => $response['paymentType'],
                'apiVersion' => $response['apiVersion'],
                'signature' => $response['signature']
            );
    
            if(!$this->checkSignature($this->config->get('payment_paybylink_hash'), $pbl_data)) {
                exit('Wrong checksum');
            }

	        $this->load->model('checkout/order');
	
	        $order_info = $this->model_checkout_order->getOrder($response['control']);
    
            $newOrderStatus = $this->config->get('payment_paybylink_order_status_id');
			if($order_info['order_status_id'] != $newOrderStatus){
				$this->model_checkout_order->addOrderHistory($response['control'], $newOrderStatus, 'Płatność została przyjęta przez system Paybylink', false);
			}
    
            ob_clean();
            exit('OK');
        } else {
            exit('Empty response');
        }   
    }
    
    public function returnOk()
    {
        $this->response->redirect(HTTPS_SERVER . 'index.php?route=checkout/success');
    }

    private function checkSignature($secret, $data){
        $sign = sprintf("%s|%s|%s|%s|%.2f|%d|%s|%d", $secret, $data['transactionId'], $data['control'], $data['email'], $data['amountPaid'], $data['notificationAttempt'], $data['paymentType'], $data['apiVersion']);
        $hash = hash('sha256', $sign);
        return $hash == $data['signature'];
    }
	
}